Imports System.Drawing
Imports System.Drawing.Drawing2D
Imports System.Windows.Forms
Public Class InfoBox
    Inherits Control

#Region " Variables/Propertys "

    Private _usedNode As VisibleNode
    Private _drawingArea As Rectangle
    Private _mouseDownTime As DateTime
    Private _bitmap As Bitmap
    Private _mode As ContentDisplayModes = ContentDisplayModes.normal
    Private _dataAvailable As Boolean = False

    Public BoxNumber As Integer
    Public Shared IL As ImageList

    Public Event XMouseDown(ByVal BoxNr As Integer)
    Public Event XMouseMove(ByVal BoxNr As Integer)
    Public Event DataChanged()

    Public ReadOnly Property DataAvailable() As Boolean
        Get
            Return _dataAvailable
        End Get
    End Property
    Public ReadOnly Property ActSize() As Size
        Get
            Return Me.Size
        End Get
    End Property
    Public ReadOnly Property usedNode() As VisibleNode
        Get
            Return _usedNode
        End Get
    End Property
    Public Property SelectionMode() As ContentDisplayModes
        Get
            Return _mode
        End Get
        Set(ByVal Value As ContentDisplayModes)
            If _mode <> Value Then
                _mode = Value
                _bitmap = Nothing
                Invalidate()
            End If
        End Set
    End Property

    Public Sub InsertNode(ByVal Node As VisibleNode)
        _usedNode = Node
        If Not IsNothing(_usedNode) Then
            _dataAvailable = True
        Else
            _dataAvailable = False
        End If
        RaiseEvent DataChanged()
        _bitmap = Nothing
        Invalidate()
    End Sub
    Public Sub RemoveNode()
        _usedNode = Nothing
        Me.Text = ""
        _dataAvailable = False
        _bitmap = Nothing
        Invalidate()
        RaiseEvent DataChanged()
    End Sub

#End Region

    Public Sub New()
        MyBase.New()

        SetStyle(ControlStyles.DoubleBuffer, True)
        SetStyle(ControlStyles.UserPaint, True)
        SetStyle(ControlStyles.AllPaintingInWmPaint, True)

        _drawingArea = New Rectangle(0, 0, 10, 10)
    End Sub

#Region " Mouse Event Handlers "

    Private Sub InfoBox_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseDown
        If Not e.Button = Windows.Forms.MouseButtons.Left Then Exit Sub
        If Me.Enabled Then
            Me.SelectionMode = ContentDisplayModes.selected
            _mouseDownTime = Now
            RaiseEvent XMouseDown(Me.BoxNumber)
        End If
    End Sub
    Private Sub InfoBox_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseMove
        If Not e.Button = Windows.Forms.MouseButtons.Left Then Exit Sub
        If Me.SelectionMode = ContentDisplayModes.selected Then
            If Now > _mouseDownTime.AddMilliseconds(250) Then RaiseEvent XMouseMove(Me.BoxNumber)
        End If
    End Sub

#End Region

#Region " Drawing Methods "

    Private Sub DrawControl(ByVal graphics As Graphics)
        Dim g As Graphics = Nothing
        If IsNothing(_bitmap) Then
            _bitmap = New Bitmap(Me.Width, Me.Height)
            g = Drawing.Graphics.FromImage(_bitmap)
            Try
                Dim SelRA As Rectangle = New Rectangle(4, 4, _bitmap.Width - 8, _bitmap.Height - 8)

                If Me.Enabled Then
                    If Me._dataAvailable Then
                        DrawSurface(g)
                    Else
                        Dim Backbrush As SolidBrush = New SolidBrush(Color.White)
                        g.FillRectangle(Backbrush, 0, 0, _bitmap.Width, _bitmap.Height)
                        Backbrush.Dispose()
                    End If
                    Select Case Me._mode
                        Case ContentDisplayModes.selected
                            DrawSelectionRectangle(g, SelRA, Color.Violet)
                        Case ContentDisplayModes.normal
                            DrawSelectionRectangle(g, SelRA, Color.Orange)
                    End Select
                Else
                    Dim Backbrush As SolidBrush = New SolidBrush(SystemColors.Control)
                    g.FillRectangle(Backbrush, 0, 0, _bitmap.Width, _bitmap.Height)
                    Backbrush.Dispose()
                    DrawSelectionRectangle(g, SelRA, Color.FromArgb(233, 186, 100))
                End If
                Select Case Me.SelectionMode
                    Case ContentDisplayModes.dragging, ContentDisplayModes.forbidden
                        Dim LinePen As New Pen(Color.DarkBlue, 2)
                        g.DrawRectangle(LinePen, New Rectangle(1, 1, _bitmap.Width - 2, _bitmap.Height - 2))
                        LinePen.Dispose()
                    Case Else
                        ControlPaint.DrawBorder3D(g, Me.ClientRectangle, Border3DStyle.Sunken)
                End Select
            Catch ex As Exception
                System.Diagnostics.Debug.WriteLine(ex.Message)
            Finally
                g.Dispose()
            End Try
        End If
        g = graphics
        Try
            g.DrawImage(_bitmap, Me.ClientRectangle)
        Catch ex As Exception
            System.Diagnostics.Debug.WriteLine(ex.Message)
        Finally
            GC.Collect()
        End Try
    End Sub
    Private Sub DrawSelectionRectangle(ByVal g As Graphics, ByVal RA As Rectangle, ByVal xColor As Color)
        Dim xPen As New Pen(xColor, 2)
        xPen.DashStyle = DashStyle.Dash
        g.DrawRectangle(xPen, RA)
        xPen.Dispose()
    End Sub
    Private Sub DrawSurface(ByVal g As Graphics)
        Dim TextBrush As SolidBrush = New SolidBrush(Color.DarkBlue)
        Select Case Me._mode
            Case ContentDisplayModes.normal, ContentDisplayModes.selected
                Dim Brsh As New LinearGradientBrush(_drawingArea, Color.White, Color.Teal, LinearGradientMode.BackwardDiagonal)
                g.FillRectangle(Brsh, _drawingArea)
                Brsh.Dispose()
            Case ContentDisplayModes.dragging
                Dim Brsh As New SolidBrush(Color.LightYellow)
                g.FillRectangle(Brsh, _drawingArea)
                Brsh.Dispose()
            Case ContentDisplayModes.floating, ContentDisplayModes.forbidden
                Dim Brsh As New HatchBrush(HatchStyle.Trellis, Color.LightYellow, Color.Red)
                g.FillRectangle(Brsh, _drawingArea)
                Brsh.Dispose()
        End Select
        If _drawingArea.Width > _drawingArea.Height Then
            g.DrawImage(IL.Images(_usedNode.SelectedImageIndex), 5, 4)
            g.DrawString(_usedNode.NodeData, Font, TextBrush, New RectangleF(25, 5, _drawingArea.Width - 30, _drawingArea.Height - 5))
        Else
            Dim Img As Image = IL.Images(_usedNode.SelectedImageIndex).Clone
            Img.RotateFlip(RotateFlipType.Rotate270FlipNone)
            g.DrawImage(Img, 5, 5)
            Img.Dispose()

            Dim txtWidth As Integer = g.MeasureString(_usedNode.NodeData, Font).ToSize.Width
            Dim drawFormat As New StringFormat()
            drawFormat.FormatFlags = StringFormatFlags.DirectionVertical

            Dim rotatePoint As New PointF(_drawingArea.Width \ 2, 25 + txtWidth \ 2)

            Dim gs As GraphicsState = g.Save
            Dim myMatrix As New Matrix()
            myMatrix.RotateAt(180, rotatePoint, MatrixOrder.Append)
            g.Transform = myMatrix
            g.DrawString(_usedNode.NodeData, Font, TextBrush, 0, 25, drawFormat)
            g.Restore(gs)
            myMatrix.Dispose()
        End If
        TextBrush.Dispose()
    End Sub

#End Region

#Region " Overrides "

    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(e)
        DrawControl(e.Graphics)
    End Sub
    Protected Overrides Sub OnResize(ByVal e As System.EventArgs)
        'berschreiben der Methode OnResize
        If Me.Width < 20 Then Me.Width = 20
        If Me.Height < 20 Then Me.Height = 20
        _drawingArea = New Rectangle(2, 2, Me.Size.Width - 4, Me.Size.Height - 4)
        _bitmap = Nothing
        Invalidate()
        MyBase.OnResize(e)
    End Sub
    Public Overrides Sub Refresh()
        'berschreiben der Methode Refresh
        MyBase.Refresh()
        Me.Invalidate()
    End Sub
    Protected Overrides Sub OnEnabledChanged(ByVal e As System.EventArgs)
        'berschreiben der Methode OnEnabledChanged
        MyBase.OnEnabledChanged(e)
        _bitmap = Nothing
        Invalidate()
    End Sub

#End Region

End Class




