Imports System.Drawing
Imports System.Drawing.Drawing2D
Imports System.Windows.Forms

Public Class InfoParkBox
    Inherits Control

#Region " Variables/Propertys "

    Private _itemHeight As Integer = 24
    Private _maxItems As Integer
    Private _list As New ArrayList()
    Private _usedNode As VisibleNode
    Private _mouseDownTime As DateTime
    Private _bitmap As Bitmap
    Private _mode As ContentDisplayModes = ContentDisplayModes.normal

    Public m_dataAvailable As Boolean = False
    Public BoxNumber As Integer
    Public IL As ImageList

    Public Event xMousemove()

    Public ReadOnly Property DataAvailable() As Boolean
        Get
            Return m_dataAvailable
        End Get
    End Property
    Public ReadOnly Property ActSize() As Size
        Get
            Dim s As New Size(Me.Width, _itemHeight)
            Return s
        End Get
    End Property
    Public ReadOnly Property usedNode() As VisibleNode
        Get
            Return _usedNode
        End Get
    End Property
    Public Property SelectionMode() As ContentDisplayModes
        Get
            Return _mode
        End Get
        Set(ByVal Value As ContentDisplayModes)
            If _mode <> Value Then
                _mode = Value
                _bitmap = Nothing
                Invalidate()
            End If
        End Set
    End Property

    Public Sub InsertNode(ByVal Node As VisibleNode)
        If _list.Contains(Node) Then
            _list.Remove(Node)
            _list.Insert(0, Node)
        Else
            _list.Insert(0, Node)
        End If
        _usedNode = Node
        If _list.Count > _maxItems Then
            _list.RemoveAt(_maxItems)
        End If
        m_dataAvailable = True
        _bitmap = Nothing
        Invalidate()
    End Sub
    Public Sub RemoveNode()
        _list.Remove(_usedNode)
        _usedNode = Nothing
        m_dataAvailable = False
        _bitmap = Nothing
        Invalidate()
    End Sub

#End Region

    Public Sub New()
        MyBase.New()

        SetStyle(ControlStyles.DoubleBuffer, True)
        SetStyle(ControlStyles.UserPaint, True)
        SetStyle(ControlStyles.AllPaintingInWmPaint, True)

    End Sub

#Region " Mouse Event Handlers "

    Private Sub InfoParkbox_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseMove
        If e.Button = Windows.Forms.MouseButtons.Left And (Now > _mouseDownTime.AddMilliseconds(250)) Then
            If Me.SelectionMode = ContentDisplayModes.itemselected Then
                RaiseEvent xMousemove()
            End If
        End If
    End Sub
    Private Sub InfoParkbox_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseDown
        If Not e.Button = Windows.Forms.MouseButtons.Left Then Exit Sub
        _mouseDownTime = Now
        Dim mousepoint As Point = New Point(e.X, e.Y)
        Dim AktuellePos As Integer = (e.Y - 2) \ _itemHeight
        If AktuellePos + 1 <= _list.Count Then
            _usedNode = _list.Item(AktuellePos)

            If Me.SelectionMode <> ContentDisplayModes.itemselected Then
                Me.SelectionMode = ContentDisplayModes.itemselected
            Else
                _bitmap = Nothing
                Invalidate()
            End If

        Else
            _usedNode = Nothing
            Me.SelectionMode = ContentDisplayModes.selected
        End If

    End Sub

#End Region

#Region " Drawing Methods "

    Private Sub DrawControl(ByVal graphics As Graphics)
        Dim g As Graphics = Nothing
        Dim I As Integer
        If IsNothing(_bitmap) Then
            _bitmap = New Bitmap(Me.Width, Me.Height)
            g = Drawing.Graphics.FromImage(_bitmap)
            Try
                Dim Backbrush As SolidBrush
                If Me.Enabled Then
                    Backbrush = New SolidBrush(Color.White)
                    g.FillRectangle(Backbrush, 0, 0, _bitmap.Width, _bitmap.Height)
                Else
                    Backbrush = New SolidBrush(SystemColors.Control)
                    g.FillRectangle(Backbrush, 0, 0, _bitmap.Width, _bitmap.Height)
                End If
                Backbrush.Dispose()
                If _list.Count > 0 Then
                    For I = 0 To _list.Count - 1
                        DrawItem(g, I)
                    Next
                End If
                Dim SelRA As Rectangle = New Rectangle(4, 4, _bitmap.Width - 8, _bitmap.Height - 8)
                If Me.Enabled Then
                    If Me._mode = ContentDisplayModes.selected Then
                        DrawSelectionRectangle(g, SelRA, Color.Violet)
                    ElseIf Me._mode = ContentDisplayModes.itemselected Then
                        DrawSelectionRectangle(g, SelRA, Color.Violet)
                        If Me.m_dataAvailable Then
                            I = _list.IndexOf(_usedNode)
                            DrawItem(g, I)
                            DrawSelectionRectangle(g, New Rectangle(4, 4 + (I * _itemHeight), _bitmap.Width - 8, _itemHeight - 4), Color.Orange)
                        End If
                    Else
                        DrawSelectionRectangle(g, SelRA, Color.Orange)
                    End If
                Else
                    DrawSelectionRectangle(g, SelRA, Color.FromArgb(233, 186, 100))
                End If

                ControlPaint.DrawBorder3D(g, Me.ClientRectangle, Border3DStyle.Sunken)

            Catch ex As Exception
                System.Diagnostics.Debug.WriteLine(ex.Message)
            Finally
                g.Dispose()
            End Try
        End If
        g = graphics
        Try
            g.DrawImage(_bitmap, Me.ClientRectangle)
        Catch ex As Exception
            System.Diagnostics.Debug.WriteLine(ex.Message)
        Finally
            GC.Collect()
        End Try
    End Sub
    Private Sub DrawSelectionRectangle(ByVal g As Graphics, ByVal RA As Rectangle, ByVal xColor As Color)
        Dim xPen As New Pen(xColor, 2)
        xPen.DashStyle = DashStyle.Dash
        g.DrawRectangle(xPen, RA)
        xPen.Dispose()
    End Sub
    Private Sub DrawItem(ByVal g As Graphics, ByVal Index As Integer)
        Dim J As Integer
        Dim TxtWidth As Integer = _bitmap.Width - 32
        Dim Txt() As String
        Dim tmptxt As String = ""
        Dim memtxt As String = ""
        Dim TextBrush As SolidBrush = New SolidBrush(Color.DarkBlue)
        Dim RAx As New Rectangle(2, 2 + (Index * _itemHeight), _bitmap.Width - 4, _itemHeight)
        Dim Brsh As New LinearGradientBrush(RAx, Color.White, Color.Teal, LinearGradientMode.BackwardDiagonal)
        g.FillRectangle(Brsh, RAx)
        Brsh.Dispose()
        Dim Node As VisibleNode = _list.Item(Index)
        g.DrawImage(IL.Images(Node.SelectedImageIndex), 5, RAx.Top + 4)
        Txt = Node.NodeData.Split(" ")
        For J = 0 To Txt.GetUpperBound(0)
            If J = 0 Then
                tmptxt = Txt(0)
            Else
                tmptxt = tmptxt & " " & Txt(J)
            End If
            If g.MeasureString(tmptxt, Me.Font).ToSize.Width > TxtWidth Then
                Exit For
            Else
                memtxt = tmptxt
            End If
        Next
        g.DrawString(memtxt, Font, TextBrush, New RectangleF(25, RAx.Top + 5, RAx.Width - 25, RAx.Height - 8))
        TextBrush.Dispose()
    End Sub

#End Region

#Region " Overrides "

    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(e)
        DrawControl(e.Graphics)
    End Sub
    Protected Overrides Sub OnResize(ByVal eventargs As System.EventArgs)
        _maxItems = (Me.Size.Height - 4) \ _itemHeight
        If _list.Count > 0 Then
            Dim I As Integer
            If _maxItems < _list.Count Then
                For I = _list.Count To _maxItems Step -1
                    _list.RemoveAt(I - 1)
                Next
            End If
        End If
        _bitmap = Nothing
        Invalidate()
    End Sub
    Public Overrides Sub Refresh()
        MyBase.Refresh()
        Me.Invalidate()
    End Sub

#End Region

End Class
